﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;

namespace WiimoteClient
{
    class Client
    {
        private Socket clientSocket;
        private byte[] receiveBuffer = new byte[100];

        public bool wiimote=false, nunchuck=false, motionplus=false;

        public string[] wiimoteButtons;
        public string[] nunchuckButtons;

        public double[] wiimoteAccel = new double[3];
        public double[] motionPlusValues = new double[3];
        public double[] nunchuckAccel = new double[3];
        public double[] nunchuckJoystick = new double[2];
        public double[] nunchuckJoystickSmooth = new double[2];
        public double[] wiimoteIR = new double[2];


        public bool initialized = false;

        public Client()
        {
            initialized = false;
        }

        public void connect()
        {
            try
            {
                clientSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP);
                IPAddress serverAddr = IPAddress.Parse("127.0.0.1");
                IPEndPoint serverMachine = new IPEndPoint(serverAddr, 8881);

                clientSocket.Connect(serverMachine);

                send(Messages.Begin);
                string response = receive();
                if (response == Messages.ACK)
                {
                    send(Messages.WiimoteConected);
                    response = receive();
                    string[] temp = response.Split(new char[] { ',' });
                    wiimote = temp[2] == "T";

                    send(Messages.NunchuckConnected);
                    response = receive();
                    string[] temp1 = response.Split(new char[] { ',' });
                    nunchuck = temp1[2] == "T";

                    send(Messages.MotionPlusConnected);
                    response = receive();
                    string[] temp2 = response.Split(new char[] { ',' });
                    motionplus = temp2[2] == "T";
                }

                initialized = true;
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
                initialized = false;
            }
        }

        public void send(string msg)
        {
            try
            {
                msg += "\n";
                //Encode a message
                byte[] sendBuffer = Encoding.ASCII.GetBytes(msg);
                clientSocket.Send(sendBuffer);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
            }
        }

        public string receive()
        {
            string message = "";
            try
            {
                clientSocket.Receive(receiveBuffer);
                message = Encoding.Default.GetString(receiveBuffer);
                //Remove the tailing '\0's after the '\n' token, caused by the buffer size
                int ixEnd = message.IndexOf('\n');
                message = message.Remove(ixEnd);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
            }

            return message;
        }

        public void disconnect()
        {
            send(Messages.End);
            string response = receive();
            if (response == Messages.ACK)
            {
                //Wait for server to close this connection
                System.Threading.Thread.Sleep(500);
            }
            else
            {
                Console.WriteLine("Client failed to disconnect.\n");
            }
        }

        public void update()
        {

            syncButtonState();
            syncJoystick();
            syncNunchuck();
            syncWiimote();
            syncIR();
            syncMotionPlus();
        }

        public void syncButtonState()
        {
            send(Messages.WiimoteButtons);
            string response = receive();
            wiimoteButtons = response.Split(new char[] { ',' });

            if (nunchuck)
            {
                send(Messages.NunchuckButtons);
                response = receive();
                nunchuckButtons = response.Split(new char[] { ',' });
            }
        }

        private void syncWiimote()
        {
            send(Messages.WiimoteAcceleration);
            string response = receive();
            string[] xyz = response.Split(new char[] { ',' });
            //Skip "[message_type],,"
            wiimoteAccel[0] = System.Convert.ToDouble(xyz[2]);            //Get X axis acceleration
            wiimoteAccel[1] = System.Convert.ToDouble(xyz[3]);            //Get Y axis acceleration
            wiimoteAccel[2] = System.Convert.ToDouble(xyz[4]);            //Get Z axis acceleration
        }

        private void syncJoystick()
        {
            send(Messages.NunchuckJoystick);
            string response = receive();
            string[] xy = response.Split(new char[] { ',' });
            
            nunchuckJoystick[0] = System.Convert.ToDouble(xy[2]);            //Get X axis acceleration
            nunchuckJoystick[1] = System.Convert.ToDouble(xy[3]);            //Get Y axis acceleration
        }

        private void syncNunchuck()
        {
            send(Messages.NunchuckAcceleration);
            string response = receive();
            string[] xyz = response.Split(new char[] { ',' });
            
            nunchuckAccel[0] = System.Convert.ToDouble(xyz[2]);            //Get X axis acceleration
            nunchuckAccel[1] = System.Convert.ToDouble(xyz[3]);            //Get Y axis acceleration
            nunchuckAccel[2] = System.Convert.ToDouble(xyz[4]);            //Get Z axis acceleration
        }

        private void syncMotionPlus()
        {
            send(Messages.MotionPlus_Acceleration);
            string response = receive();
            string[] xyz = response.Split(new char[] { ',' });
            
            motionPlusValues[0] = System.Convert.ToDouble(xyz[2]);            //Get X axis 
            motionPlusValues[1] = System.Convert.ToDouble(xyz[3]);            //Get Y axis 
            motionPlusValues[2] = System.Convert.ToDouble(xyz[4]);            //Get Z axis 
        }


        private void syncIR()
        {
            send(Messages.WiimoteIR);
            string response = receive();
            string[] xy = response.Split(new char[] { ',' });
            
            wiimoteIR[0] = System.Convert.ToDouble(xy[2]);            //Get X axis acceleration
            wiimoteIR[1] = System.Convert.ToDouble(xy[3]);            //Get Y axis acceleration
        }
    }
}
