﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.IO;
using System.Net;

namespace WiimoteServer
{
    class WServer
    {
        private Socket server;
        private Socket clientSocket;

        private byte[] receiveBuffer, sendBuffer;
        private ASCIIEncoding encoder;

        private WiimoteInterface wInterface;

        public WServer()
        {
            wInterface = WiimoteInterface.Instance;
            clientSocket = null;
            
            receiveBuffer = new byte[100];
            sendBuffer = new byte[100];
            encoder = new ASCIIEncoding();

            server = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.IP);
            IPEndPoint local_ip = new IPEndPoint(IPAddress.Any, 8881);
            server.Bind(local_ip);
        }

        public void start()
        {
            server.Listen(5);
            Console.WriteLine("Running Wiimote Server at {0}", System.Environment.MachineName);

            while (true)
            {
                clientSocket = server.Accept();
                handleClient();
            }
        }

        private void handleClient()
        {
            string message = getRequest();
            
            if (message == Messages.Begin)
            {
                //hand shake
                sendResponse(Messages.ACK);
                Console.WriteLine("Connection Received and Confirmed");
                while (message != Messages.End)
                {
                    message = getRequest();

                    string response = wInterface.getValue(message);
                    sendResponse(response);
                }

                //good bye
                sendResponse(Messages.ACK);
                Console.WriteLine("Terminating connection");
            }

        }

        private string getRequest()
        {
            string message = "";
            try
            {
                //Get message from client
                clientSocket.Receive(receiveBuffer);
                message = encoder.GetString(receiveBuffer);

                //parse message: all messages must end with a '\n'
                int index = message.IndexOf('\n');
                message = message.Remove(index);
            }
            catch(Exception e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
            }

            return message;
        }

        private void sendResponse(string response)
        {
            try
            {
                response += "\n";
                sendBuffer = encoder.GetBytes(response);
                clientSocket.Send(sendBuffer);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
            }
        }
    }
}
