﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WiimoteLib;

namespace WiimoteServer
{
    internal class WiimoteInterface
    {
        private WiimoteCollection allWiimotes;
        private Wiimote wiimote;

        public bool NunchuckConnected {get; set;}
        public bool MotionPlusConnected { get; set; }
        public bool WiimoteConnected { get; set; }

        private static WiimoteInterface _instance = null;
        public static WiimoteInterface Instance
        {
            get
            {
                if (_instance == null) _instance = new WiimoteInterface();
                return _instance;
            }
        }

        protected WiimoteInterface()
        {
            try
            {
                allWiimotes = new WiimoteCollection();
                allWiimotes.FindAllWiimotes();

                bool[] leds = new bool[] { false, false, false, false };
                for (int i = 0; i < allWiimotes.Count; ++i)
                {
                    leds[0] = leds[1] = leds[2] = leds[3] = false;
                    leds[i] = true;
                    Wiimote wm = allWiimotes[i];
                    wm.Connect();
                    wm.InitializeMotionPlus();

                    wm.SetReportType(InputReport.IRExtensionAccel, IRSensitivity.Maximum, true);
                    wm.SetLEDs(leds[0], leds[1], leds[2], leds[3]);

                    wiimote = wm;


                    if (wiimote.WiimoteState.ExtensionType == ExtensionType.Nunchuk)
                    {
                        NunchuckConnected = true;
                        Console.WriteLine("Wiimote {0} Connected, Nunchuck Attached", i + 1);
                    }
                    else if (wiimote.WiimoteState.ExtensionType == ExtensionType.MotionPlus)
                    {
                        MotionPlusConnected = true;
                        Console.WriteLine("Wiimote {0} Connected, MotionPlus Attached", i + 1);
                    }
                    else
                        Console.WriteLine("Wiimote {0} connected", i + 1);

                    WiimoteConnected = true;
                }
            }
            catch (WiimoteNotFoundException wnfe)
            {
                Console.WriteLine("No Wiimotes Detected!!!!");
                Console.WriteLine(wnfe.StackTrace);
            }
            catch (WiimoteException we)
            {
                Console.WriteLine("Wiimote Exception!!!!");
                Console.WriteLine(we.StackTrace);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.StackTrace);
            }

        }

        public void close()
        {
            foreach (Wiimote wiimote in allWiimotes)
                wiimote.Disconnect();
        }

        public string getValue(string request)
        {
            string value = "";
            switch (request)
            {
                case Messages.WiimoteConected:
                    value = isWiimoteConnected();
                    break;
                case Messages.WiimoteButtons:
                    value = getWiimoteButtons();
                    break;
                case Messages.WiimoteAcceleration:
                    value = getWiimoteAcceleration();
                    break;
                case Messages.NunchuckConnected:
                    value = isNunchuckConnected();
                    break;
                case Messages.NunchuckAcceleration:
                    value = getNunchuckAcceleration();
                    break;
                case Messages.NunchuckButtons:
                    value = getNunchuckButtons();
                    break;
                case Messages.NunchuckJoystick:
                    value = getNunchuckJoystick();
                    break;
                case Messages.WiimoteIR:
                    value = getIRData();
                    break;
                case Messages.MotionPlusConnected:
                    value = isMotionPlusConnected();
                    break;
                case Messages.MotionPlus_Acceleration:
                    value = getMotionPlusAcceleration();
                    break;
            }

            return value;
        }


        /*
         * All messages are strings and of the format
         * "[message_type],,[value1], [value2],....., [last_value]\n"
         * 
         * float and int values are sent as they are
         * bool values are sent as 'T' or 'F'
        */

        #region Functions to Read Wiimote Data

        private string isWiimoteConnected()
        {
            string status = "WIIMOTE_CONNECTED,,";
            status += ((WiimoteConnected == true) ? "T" : "F");

            return status;
        }

        private string getWiimoteAcceleration()
        {
            string status = "WIIMOTE_ACCEL,,";
            status += wiimote.WiimoteState.AccelState.Values.X.ToString();
            status += ",";
            status += wiimote.WiimoteState.AccelState.Values.Y.ToString();
            status += ",";
            status += wiimote.WiimoteState.AccelState.Values.Z.ToString();

            return status;
        }

        private string getWiimoteButtons()
        {
            string status = "WIIMOTE_BUTTONS,,";
            status += ((wiimote.WiimoteState.ButtonState.A == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.B == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Up == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Down == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Left == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Right == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Plus == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Minus == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.One == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Two == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.ButtonState.Home == true) ? "T" : "F");

            return status;
        }

        private string getIRData()
        {
            string status = "IR_VALUES,,";
            status += wiimote.WiimoteState.IRState.Midpoint.X.ToString();
            status += ",";
            status += wiimote.WiimoteState.IRState.Midpoint.Y.ToString();

            return status;
        }

        #endregion

        #region Functions to read MotionPlus data

        private string isMotionPlusConnected()
        {
            string status = "MOTIONPLUS_CONNECTED,,";
            status += ((MotionPlusConnected == true) ? "T" : "F");

            return status;
        }

        private string getMotionPlusAcceleration()
        {
            string status = "MOTIONPLUS_ACCEL,,";
            status += wiimote.WiimoteState.MotionPlusState.RawValues.X.ToString();
            status += ",";
            status += wiimote.WiimoteState.MotionPlusState.RawValues.Y.ToString();
            status += ",";
            status += wiimote.WiimoteState.MotionPlusState.RawValues.Z.ToString();

            return status;
        }

        #endregion

        #region Functions to Read Nunchuck Data

        private string isNunchuckConnected()
        {
            string status = "NUNCHUCK_CONNECTED,,";
            status += ((NunchuckConnected == true) ? "T" : "F");

            return status;
        }

        private string getNunchuckAcceleration()
        {
            string status = "NUNCHUCK_ACCEL,,";
            status += wiimote.WiimoteState.NunchukState.AccelState.Values.X.ToString();
            status += ",";
            status += wiimote.WiimoteState.NunchukState.AccelState.Values.Y.ToString();
            status += ",";
            status += wiimote.WiimoteState.NunchukState.AccelState.Values.Z.ToString();

            return status;
        }

        private string getNunchuckJoystick()
        {
            string status = "NUNCHUCK_JOYSTICK,,";
            status += wiimote.WiimoteState.NunchukState.Joystick.X.ToString();
            status += ",";
            status += wiimote.WiimoteState.NunchukState.Joystick.Y.ToString();

            return status;
        }

        private string getNunchuckButtons()
        {
            string status = "NUNCHUCK_BUTTONS,,";
            status += ((wiimote.WiimoteState.NunchukState.C == true) ? "T," : "F,");
            status += ((wiimote.WiimoteState.NunchukState.Z == true) ? "T" : "F");

            return status;
        }

        #endregion

    }
}
